function [G_sgwt, C_sgwt, C_sgwt_hat, sgwt] = build_sgwt_atoms(graph, nb_scales, lambda_max, lp_factor)
    %% Init
    N = grasp_nb_nodes(graph);
    grasp_start_opt_3rd_party('sgwt');
    if size(graph.Finv(:)) ~= N ^ 2
        error('The GFT is not built!');
    end
    
    if nargin < 3
        lambda_max = max(graph.eigvals);
        if nargin < 4
            lp_factor = 20;
        end
    end

    %% Code
    sgwt = sgwt_filter_design(lambda_max, nb_scales, 'lpfactor', lp_factor);
    C_sgwt_hat = cell2mat(arrayfun(@(m) sgwt{m}(graph.eigvals'), (1:(nb_scales + 1))', 'UniformOutput', false));

    C_sgwt = graph.Finv * C_sgwt_hat';

    %% Transform
    G_sgwt = zeros(N * (nb_scales + 1) , N);
    for i = 1:N
        G_sgwt(((i - 1) * (nb_scales + 1) + 1):(i * (nb_scales + 1)), :) = (grasp_generalized_translation(graph, i) * C_sgwt)';
    end
end