%% Random Plane Sampling (Sensor Network)

% g_plane = grasp_plane_rnd(200);
% g_plane.layout = g_plane.layout / 10;  % Trick to consider a unit square (layout is on a 10x10 square)
% g_plane.distances = grasp_distances_layout(g_plane);
% g_plane.layout = g_plane.layout * 10;
% g_plane.sigma = mean(g_plane.distances(g_plane.distances > 0)) / 3;
% g_plane.A = grasp_adjacency_gaussian(g_plane, g_plane.sigma);

load('g_plane.mat', 'g_plane');
g_plane.sigma = mean(g_plane.distances(g_plane.distances > 0)) / 3;

%NOTE: Uncomment the two following lines to do the analysis using the
%optimal sigma with respect to minimizing spectral correlations
% g_plane.sigma = 0.032;

g_plane.A = grasp_adjacency_gaussian(g_plane, g_plane.sigma);
fprintf('Gauss CMP Sigma: %f\n', g_plane.sigma);

%% Some thresholding

g_plane_knn = g_plane;
g_plane_knn.KNN = 6;
g_plane_knn.A = grasp_adjacency_knn(g_plane, g_plane_knn.KNN) > 0;
g_plane_knn.A_layout = g_plane_knn.A;
g_plane_knn.sigma = mean(g_plane.distances(g_plane_knn.A > 0)) / 3;
g_plane_knn.A = g_plane_knn.A .* grasp_adjacency_gaussian(g_plane, g_plane_knn.sigma);
fprintf('Mean number of neighbors (KNN): %f\n', mean((g_plane_knn.A > 0) * ones(grasp_nb_nodes(g_plane), 1)));
fprintf('Gauss KNN Sigma: %f\n', g_plane_knn.sigma);

g_plane_thr = g_plane;
g_plane_thr.dst_max = 0.1114;
g_plane_thr.threshold = exp(-g_plane_thr.dst_max ^ 2 / (2 * g_plane.sigma ^ 2));
g_plane_thr.A = grasp_adjacency_thresh(g_plane, g_plane_thr.threshold) > 0;
g_plane_thr.A_layout = g_plane_thr.A;
g_plane_thr.sigma = mean(g_plane.distances(g_plane_thr.A > 0)) / 3;
g_plane_thr.A = g_plane_thr.A .* grasp_adjacency_gaussian(g_plane, g_plane_thr.sigma);
fprintf('Mean number of neighbors (THR): %f\n', mean((g_plane_thr.A > 0) * ones(grasp_nb_nodes(g_plane), 1)));
fprintf('Radius: %f\n', g_plane_thr.dst_max);
fprintf('Gauss THR Sigma: %f\n', g_plane_thr.sigma);

%% Local PSD from global correlation

% Unnormalized Atoms
% lpsd = @(R, G) reshape(diag(G * R * G'), size(G, 1) / size(G, 2), size(G, 2));

% Normalized Atoms
lpsd = @(R, G) reshape(diag(G * R * G') ./ diag(G * G'), size(G, 1) / size(G, 2), size(G, 2));

%% Harmonic analysis

eig_matrix = 'std_lapl';

g_plane = grasp_eigendecomposition(g_plane, 'matrix', eig_matrix);
g_plane_knn = grasp_eigendecomposition(g_plane_knn, 'matrix', eig_matrix);
g_plane_thr = grasp_eigendecomposition(g_plane_thr, 'matrix', eig_matrix);

[G_sgwt_plane, C_sgwt_plane, C_sgwt_hat_plane, sgwt_resp_plane] = build_sgwt_atoms(g_plane, 30, max(g_plane.eigvals), 20);
[G_sgwt_plane_knn, C_sgwt_plane_knn, C_sgwt_hat_plane_knn, sgwt_resp_plane_knn] = build_sgwt_atoms(g_plane_knn, 30, max(g_plane_knn.eigvals), 500000);
[G_sgwt_plane_thr, C_sgwt_plane_thr, C_sgwt_hat_plane_thr, sgwt_resp_plane_thr] = build_sgwt_atoms(g_plane_thr, 30, max(g_plane_thr.eigvals), 500000);

%% Some Signals in the Underlying Euclidean Space

% (x,y): coordinates
% (k,l): frequency (1: one period between coordinates separated by 1)
% phi: phase in [0,1]
s_exp = @(x,y,k,l,phi) exp(1i * 2 * pi * (k * x + l * y)) * exp(1i * 2 * pi * phi);
s_cos = @(x,y,k,l,phi) cos(2 * pi * (k * x + l * y + phi));

%% GMRF Model Theoretical Covariance

dx = repmat(g_plane.layout(:, 1), [1, grasp_nb_nodes(g_plane)]) / 10 - repmat(g_plane.layout(:, 1)', [grasp_nb_nodes(g_plane), 1]) / 10;
dy = repmat(g_plane.layout(:, 2), [1, grasp_nb_nodes(g_plane)]) / 10 - repmat(g_plane.layout(:, 2)', [grasp_nb_nodes(g_plane), 1]) / 10;

% Each frequency along each axis appears independently within the signal,
% hence the covariance is the sum of the covariance of each frequency and
% each axis.
exact_corr = @(freqs, psd, delta) sum(reshape(cell2mat(arrayfun(@(i) psd(i) * cos(2 * pi * freqs(i) * delta), 1:numel(psd), 'UniformOutput', false)), [size(dx, 1) size(dx, 2) numel(psd)]), 3);

% PSD of 1/f^2 to follow the GMRF model (1/lambda).
freqs = 1:floor(grasp_nb_nodes(g_plane) / 2);
psd =  1 ./ freqs .^ 2;
Sigma_theory = exact_corr(freqs, psd, dx) + exact_corr(freqs, psd, dy);

%% Graph Learning from the GMRF model above

nb_step_optim = 100;

Lapl_learnt     = estimate_cgl(Sigma_theory, ones(grasp_nb_nodes(g_plane)) - eye(grasp_nb_nodes(g_plane)), 0, 1e-4, 1e-6, nb_step_optim);
Lapl_learnt_knn = estimate_cgl(Sigma_theory, g_plane_knn.A > 0, 0, 1e-4, 1e-6, nb_step_optim);
Lapl_learnt_thr = estimate_cgl(Sigma_theory, g_plane_thr.A > 0, 0, 1e-4, 1e-6, nb_step_optim);

%% Learnt Graphs Harmonic Analysis

g_learnt = g_plane;
g_learnt.A = diag(diag(Lapl_learnt)) - Lapl_learnt;
g_learnt = grasp_eigendecomposition(g_learnt, 'matrix', eig_matrix);
[G_sgwt_learnt, C_sgwt_learnt, C_sgwt_hat_learnt, sgwt_resp_learnt] = build_sgwt_atoms(g_learnt, 30, max(g_learnt.eigvals), 500);

g_learnt_knn = g_plane_knn;
g_learnt_knn.A = diag(diag(Lapl_learnt_knn)) - Lapl_learnt_knn;
g_learnt_knn = grasp_eigendecomposition(g_learnt_knn, 'matrix', eig_matrix);
[G_sgwt_learnt_knn, C_sgwt_learnt_knn, C_sgwt_hat_learnt_knn, sgwt_resp_learnt_knn] = build_sgwt_atoms(g_learnt_knn, 30, max(g_learnt_knn.eigvals), 10000);

g_learnt_thr = g_plane_thr;
g_learnt_thr.A = diag(diag(Lapl_learnt_thr)) - Lapl_learnt_thr;
g_learnt_thr = grasp_eigendecomposition(g_learnt_thr, 'matrix', eig_matrix);
[G_sgwt_learnt_thr, C_sgwt_learnt_thr, C_sgwt_hat_learnt_thr, sgwt_resp_learnt_thr] = build_sgwt_atoms(g_learnt_thr, 30, max(g_learnt_thr.eigvals), 10000);

%% Plot the graph with the connectivity pattern

fh = grasp_open_figure_name('Graph Weights');
SA_M = 0.01;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.03;
SA_MV = 0.03;
SA_SV = 0.01;

ah = grasp_subaxis(fh, 2, 3, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_plane;
g_tmp.A_layout = g_tmp.A .* (g_tmp.A > 1 / 64);  % 64 is the number of colors of the colormaps in matlab
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Gauss CMP', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_plane_knn;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Gauss KNN', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_plane_thr;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Gauss THR', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_learnt;
g_tmp.A_layout = g_tmp.A .* (g_tmp.A > 1 / 64);
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Learnt CMP', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_learnt_knn;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Learnt KNN', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_learnt_thr;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Learnt THR', 'FontSize', 20);

%% Weight VS Distance

g_tmp = g_plane_thr;
g_tmp.sigma = 0.032;
g_tmp.A = g_tmp.A .* grasp_adjacency_gaussian(g_tmp, g_tmp.sigma);

grasp_open_figure_name('Weight VS Distance');
plot(g_plane_thr.distances(g_plane_thr.A > 0), g_plane.A(g_plane_thr.A > 0), '.', 'DisplayName', 'Gauss CMP (Heuristic)', 'MarkerSize', 10);
hold on
plot(g_tmp.distances(g_tmp.A > 0), g_tmp.A(g_tmp.A > 0), '.', 'DisplayName', 'Gauss CMP (Optimal Correlation)', 'MarkerSize', 10);
plot(g_learnt_thr.distances(g_learnt_thr.A > 0), g_learnt_thr.A(g_learnt_thr.A > 0), '.', 'DisplayName', 'Learnt CMP', 'MarkerSize', 10);
hold off
lh = legend('show');
lh.FontSize = 15;
ylim([0 5]);
xlabel('Distance', 'FontSize', 15);
ylabel('Weight', 'FontSize', 15);

%% Plot the eigenvalues

grasp_open_figure_name('Eigenvalues'); clf;
plot(1:grasp_nb_nodes(g_plane), [g_plane.eigvals g_plane_knn.eigvals g_plane_thr.eigvals g_learnt.eigvals g_learnt_knn.eigvals g_learnt_thr.eigvals], 'LineWidth', 2);
legend({'Gauss CMP', 'Gauss KNN', 'Gauss THR', 'Learnt CMP', 'Learnt KNN', 'Learnt THR'}, 'FontSize', 15)
ylim([0 g_learnt.eigvals(end)]);
axes('Position', [.18 .5 .4 .4]);
plot(1:grasp_nb_nodes(g_plane), [g_plane.eigvals g_plane_knn.eigvals g_plane_thr.eigvals g_learnt.eigvals g_learnt_knn.eigvals g_learnt_thr.eigvals], 'LineWidth', 1.5);
ylim([0 .3]);
xlim([0 20]);

%% Some Fourier modes

sprintf('Minimum Fourier mode sample value: %f\n', ...
    [min(min(g_plane.Finv(:, 2:4)))...
    min(min(g_plane_knn.Finv(:, 2:4)))...
    min(min(g_plane_thr.Finv(:, 2:4)))...
    min(min(g_learnt.Finv(:, 2:4)))...
    min(min(g_learnt_knn.Finv(:, 2:4)))...
    min(min(g_learnt_thr.Finv(:, 2:4)))]);

sprintf('Maximum Fourier mode sample value: %f\n', ...
    -[min(min(-g_plane.Finv(:, 2:4)))...
    min(min(-g_plane_knn.Finv(:, 2:4)))...
    min(min(-g_plane_thr.Finv(:, 2:4)))...
    min(min(-g_learnt.Finv(:, 2:4)))...
    min(min(-g_learnt_knn.Finv(:, 2:4)))...
    min(min(-g_learnt_thr.Finv(:, 2:4)))]);

fh = grasp_open_figure_name('Mode #2,#3,#4 Comparison');
gft_freqs = [2 3 4];
% gft_freqs = [5 6 7];
% gft_freqs = [8 9 10];
% freqs = [198 199 200];
SA_M = 0.01;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.03;
SA_MV = 0.03;
SA_SV = 0.01;
for cur_freq = 1:numel(gft_freqs)
    l = gft_freqs(cur_freq);
    
    ah = grasp_subaxis(fh, 3, 7, (cur_freq - 1) * 7 + 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
    grasp_show_graph(ah, g_plane, 'show_edges', false, 'node_values', g_plane.Finv(:, l), 'value_scale', 0.3 * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if l == 2
        title('Gauss CMP', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_plane.Finv(:, l)' * g_plane.L * g_plane.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 7, (cur_freq - 1) * 7 + 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
    grasp_show_graph(ah, g_plane_knn, 'node_values', g_plane_knn.Finv(:, l), 'value_scale', 0.3 * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if l == 2
        title('Gauss KNN', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_plane_knn.Finv(:, l)' * g_plane.L * g_plane_knn.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 7, (cur_freq - 1) * 7 + 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
    grasp_show_graph(ah, g_plane_thr, 'node_values', g_plane_thr.Finv(:, l), 'value_scale', 0.3 * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if l == 2
        title('Gauss THR', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_plane_thr.Finv(:, l)' * g_plane.L * g_plane_thr.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 7, (cur_freq - 1) * 7 + 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
    grasp_show_graph(ah, g_learnt, 'show_edges', false, 'node_values', g_learnt.Finv(:, l), 'value_scale', 0.3 * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if l == 2
        title('Learnt CMP', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_learnt.Finv(:, l)' * g_plane.L * g_learnt.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 7, (cur_freq - 1) * 7 + 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
    grasp_show_graph(ah, g_learnt_knn, 'node_values', g_learnt_knn.Finv(:, l), 'value_scale', 0.3 * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if l == 2
        title('Learnt KNN', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_learnt_knn.Finv(:, l)' * g_plane.L * g_learnt_knn.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 7, (cur_freq - 1) * 7 + 7, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
    grasp_show_graph(ah, g_learnt_thr, 'node_values', g_learnt_thr.Finv(:, l), 'value_scale', 0.3 * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if l == 2
        title('Learnt THR', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_learnt_thr.Finv(:, l)' * g_plane.L * g_learnt_thr.Finv(:, l)), 'FontSize', 15);
end

%% Add some close continuous Fourier modes

g_tmp = grasp_non_directed_grid(30);

ah = grasp_subaxis(fh, 3, 7, 1, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.03, 'MV', 0.03, 'SV', 0.01);
t = pi / 8;
grasp_show_graph(ah, g_tmp, 'show_edges', false, 'node_values', s_cos(g_tmp.layout(:, 1) / 10, g_tmp.layout(:, 2) / 10, .25 * cos(t), -.25 * sin(t), 0.68), 'value_scale', [-1 1]);
grasp_set_blue_red_colormap(ah);
title('Continuous', 'FontSize', 20);
xlabel('(0.25, \pi/8, 0.68)', 'FontSize', 15);


ah = grasp_subaxis(fh, 3, 7, 8, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.03, 'MV', 0.03, 'SV', 0.01);
t = 5 * pi / 8;
grasp_show_graph(ah, g_tmp, 'show_edges', false, 'node_values', s_cos(g_tmp.layout(:, 1) / 10, g_tmp.layout(:, 2) / 10, .25 * cos(t), -.25 * sin(t), 0.43), 'value_scale', [-1 1]);
grasp_set_blue_red_colormap(ah);
xlabel('(0.25, 5\pi/8, 0.43)', 'FontSize', 15);

ah = grasp_subaxis(fh, 3, 7, 15, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.03, 'MV', 0.03, 'SV', 0.01);
t = 0;
f = s_cos(g_tmp.layout(:, 1) / 10, g_tmp.layout(:, 2) / 10, .5 * cos(t), -.5 * sin(t), 0);
t = pi / 2;
f = f .* s_cos(g_tmp.layout(:, 1) / 10, g_tmp.layout(:, 2) / 10, .5 * cos(t), -.5 * sin(t), 0);
grasp_show_graph(ah, g_tmp, 'show_edges', false, 'node_values', f, 'value_scale', [-1 1]);
grasp_set_blue_red_colormap(ah);
xlabel('(0.5, 0, 0) -- (0.5, pi / 2, 0)', 'FontSize', 15);

%% Spectral Correlation & Correlation Coefficient Matrices for the various graphs

Gamma_gauss = g_plane.F * Sigma_theory * g_plane.Finv;
gamma_gauss = diag(Gamma_gauss);
Gamma_gauss_coeff = diag(1 ./ sqrt(gamma_gauss)) * Gamma_gauss * diag(1 ./ sqrt(gamma_gauss));

Gamma_gauss_knn = g_plane_knn.F * Sigma_theory * g_plane_knn.Finv;
gamma_gauss_knn = diag(Gamma_gauss_knn);
Gamma_gauss_knn_coeff = diag(1 ./ sqrt(gamma_gauss_knn)) * Gamma_gauss_knn * diag(1 ./ sqrt(gamma_gauss_knn));

Gamma_gauss_thr = g_plane_thr.F * Sigma_theory * g_plane_thr.Finv;
gamma_gauss_thr = diag(Gamma_gauss_thr);
Gamma_gauss_thr_coeff = diag(1 ./ sqrt(gamma_gauss_thr)) * Gamma_gauss_thr * diag(1 ./ sqrt(gamma_gauss_thr));

Gamma_learnt = g_learnt.F * Sigma_theory * g_learnt.Finv;
gamma_learnt = diag(Gamma_learnt);
Gamma_learnt_coeff = diag(1 ./ sqrt(gamma_learnt)) * Gamma_learnt * diag(1 ./ sqrt(gamma_learnt));

Gamma_learnt_knn = g_learnt_knn.F * Sigma_theory * g_learnt_knn.Finv;
gamma_learnt_knn = diag(Gamma_learnt_knn);
Gamma_learnt_knn_coeff = diag(1 ./ sqrt(gamma_learnt_knn)) * Gamma_learnt_knn * diag(1 ./ sqrt(gamma_learnt_knn));

Gamma_learnt_thr = g_learnt_thr.F * Sigma_theory * g_learnt_thr.Finv;
gamma_learnt_thr = diag(Gamma_learnt_thr);
Gamma_learnt_thr_coeff = diag(1 ./ sqrt(gamma_learnt_thr)) * Gamma_learnt_thr * diag(1 ./ sqrt(gamma_learnt_thr));


fh = grasp_open_figure_name('Theoretical Pearson Correlations');
SA_M = 0.0;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.0;

grasp_subaxis(gcf, 1, 7, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT);
C = diag(1 ./ sqrt(diag(Sigma_theory))) * Sigma_theory * diag(1 ./ sqrt(diag(Sigma_theory)));
imshow(C);
caxis([0 1/2]);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(C(:))) - 1 / grasp_nb_nodes(g_plane)), 'FontSize', 15);
title('Original Data', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT);
imshow(Gamma_gauss_coeff);
caxis([0 1/2]);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_gauss_coeff(:))) - 1 / grasp_nb_nodes(g_plane)), 'FontSize', 15);
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT);
imshow(Gamma_gauss_knn_coeff);
caxis([0 1/2]);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_gauss_knn_coeff(:))) - 1 / grasp_nb_nodes(g_plane)), 'FontSize', 15);
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT);
imshow(Gamma_gauss_thr_coeff);
caxis([0 1/2]);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_gauss_thr_coeff(:))) - 1 / grasp_nb_nodes(g_plane)), 'FontSize', 15);
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT);
imshow(Gamma_learnt_coeff);
caxis([0 1/2]);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_learnt_coeff(:))) - 1 / grasp_nb_nodes(g_plane)), 'FontSize', 15);
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT);
imshow(Gamma_learnt_knn_coeff);
caxis([0 1/2]);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_learnt_knn_coeff(:))) - 1 / grasp_nb_nodes(g_plane)), 'FontSize', 15);
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 7, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT);
imshow(Gamma_learnt_thr_coeff);
caxis([0 1/2]);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_learnt_thr_coeff(:))) - 1 / grasp_nb_nodes(g_plane)), 'FontSize', 15);
title('Learnt THR', 'FontSize', 20);

%% Compute correlation against various values of sigma

sigmas = logspace(-5, log10(2), 500);

means_corr_coeff = zeros(numel(sigmas), 1);
means_corr_coeff_knn = zeros(numel(sigmas), 1);
means_corr_coeff_thr = zeros(numel(sigmas), 1);

for cur_sig_ind = 1:numel(sigmas)
    cur_sig = sigmas(cur_sig_ind);
    
    cur_plane = g_plane;
    cur_plane.A = grasp_adjacency_gaussian(g_plane, cur_sig);
    cur_plane = grasp_eigendecomposition(cur_plane, 'matrix', eig_matrix);
    
    cur_plane_knn = g_plane_knn;
    cur_plane_knn.A = g_plane_knn.A > 0;
    cur_plane_knn.A_layout = cur_plane_knn.A;
    cur_plane_knn.A = cur_plane_knn.A .* grasp_adjacency_gaussian(cur_plane, cur_sig);
    cur_plane_knn = grasp_eigendecomposition(cur_plane_knn, 'matrix', eig_matrix);

    cur_plane_thr = g_plane_thr;
    cur_plane_thr.A = g_plane_thr.A > 0;
    cur_plane_thr.A_layout = cur_plane_thr.A;
    cur_plane_thr.A = cur_plane_thr.A .* grasp_adjacency_gaussian(cur_plane, cur_sig);
    cur_plane_thr = grasp_eigendecomposition(cur_plane_thr, 'matrix', eig_matrix);
    
    cur_Gamma_gauss = cur_plane.F * Sigma_theory * cur_plane.Finv;
    cur_gamma_gauss = diag(cur_Gamma_gauss);
    cur_Gamma_gauss_coeff = diag(1 ./ sqrt(cur_gamma_gauss)) * cur_Gamma_gauss * diag(1 ./ sqrt(cur_gamma_gauss));
    means_corr_coeff(cur_sig_ind) = mean(abs(cur_Gamma_gauss_coeff(:))) - 1 / grasp_nb_nodes(cur_plane);

    cur_Gamma_gauss_knn = cur_plane_knn.F * Sigma_theory * cur_plane_knn.Finv;
    cur_gamma_gauss_knn = diag(cur_Gamma_gauss_knn);
    cur_Gamma_gauss_knn_coeff = diag(1 ./ sqrt(cur_gamma_gauss_knn)) * cur_Gamma_gauss_knn * diag(1 ./ sqrt(cur_gamma_gauss_knn));
    means_corr_coeff_knn(cur_sig_ind) = mean(abs(cur_Gamma_gauss_knn_coeff(:))) - 1 / grasp_nb_nodes(cur_plane);

    cur_Gamma_gauss_thr = cur_plane_thr.F * Sigma_theory * cur_plane_thr.Finv;
    cur_gamma_gauss_thr = diag(cur_Gamma_gauss_thr);
    cur_Gamma_gauss_thr_coeff = diag(1 ./ sqrt(cur_gamma_gauss_thr)) * cur_Gamma_gauss_thr * diag(1 ./ sqrt(cur_gamma_gauss_thr));
    means_corr_coeff_thr(cur_sig_ind) = mean(abs(cur_Gamma_gauss_thr_coeff(:))) - 1 / grasp_nb_nodes(cur_plane);
end

%% Plot Correlation VS Sigma

grasp_open_figure_name('Sigma Influence');
xlimits_sigma = [10^-5 2];
semilogx(sigmas, [means_corr_coeff means_corr_coeff_knn means_corr_coeff_thr],...
    xlimits_sigma, [...
    mean(abs(Gamma_learnt_coeff(:))) - 1 / grasp_nb_nodes(g_plane) * [1 ; 1]...
    mean(abs(Gamma_learnt_knn_coeff(:))) - 1 / grasp_nb_nodes(g_plane) * [1 ; 1]...
    mean(abs(Gamma_learnt_thr_coeff(:))) - 1 / grasp_nb_nodes(g_plane) * [1 ; 1]], '-.', 'LineWidth', 1.5);
hold off
legend({'Gauss CMP', 'Gauss KNN', 'Gauss THR', 'Learnt CMP', 'Learnt KNN', 'Learnt THR'}, 'Location', 'SouthWest', 'FontSize', 15);
xlim(xlimits_sigma);
ylim([0 0.3]);
xlabel('\sigma', 'FontSize', 15);
ylabel('\mu_{offdiag}', 'FontSize', 15);

%% Graph PSD

%Note: the first graph frequency is removed otherwise log(0)=-infinity

fh = grasp_open_figure_name('Theoretical gPSD for various graphs');

gpsd_limits = [10^-3 10^2];
SA_M = 0.01;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.03;
SA_MT = 0.08;
SA_MB = 0.06;

grasp_subaxis(fh, 1, 6, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_plane.eigvals(2:end), diag(Gamma_gauss(2:end, 2:end)), ...
    g_plane.eigvals(2):0.01:g_plane.eigvals(end), 1000 ./ (g_plane.eigvals(2):0.01:g_plane.eigvals(end)) .^ (2), '--',...
    g_plane.eigvals(2):0.01:g_plane.eigvals(end), 100 ./ (g_plane.eigvals(2):0.01:g_plane.eigvals(end)) .^ (1), '-.',...
    'Linewidth', 2);
xlim([0 max(g_plane.eigvals)]);
ylim(gpsd_limits);
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_plane_knn.eigvals(2:end), diag(Gamma_gauss_knn(2:end, 2:end)), ...
    g_plane_knn.eigvals(2):0.01:g_plane_knn.eigvals(end), .4 ./ (g_plane_knn.eigvals(2):0.01:g_plane_knn.eigvals(end)) .^ (0.5), '--',...
    g_plane_knn.eigvals(2):0.01:g_plane_knn.eigvals(end), .03 ./ (g_plane_knn.eigvals(2):0.01:g_plane_knn.eigvals(end)) .^ (1), '-.',...
    'Linewidth', 2);
xlim([0 max(g_plane_knn.eigvals)]);
ylim(gpsd_limits);
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_plane_thr.eigvals(2:end), diag(Gamma_gauss_thr(2:end, 2:end)), ...
    g_plane_thr.eigvals(2):0.01:g_plane_thr.eigvals(end), .5 ./ (g_plane_thr.eigvals(2):0.01:g_plane_thr.eigvals(end)) .^ (.5), '--', ...
    g_plane_thr.eigvals(2):0.01:g_plane_thr.eigvals(end), .1 ./ (g_plane_thr.eigvals(2):0.01:g_plane_thr.eigvals(end)) .^ (1), '-.',...
    'Linewidth', 2);
xlim([0 max(g_plane_thr.eigvals)]);
ylim(gpsd_limits);
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_learnt.eigvals(2:end), diag(Gamma_learnt(2:end, 2:end)), ...
    g_learnt.eigvals(2):0.01:g_learnt.eigvals(end), 3 ./ (g_learnt.eigvals(2):0.01:g_learnt.eigvals(end)) .^ 1, '--', ...
    g_learnt.eigvals(2):0.01:g_learnt.eigvals(end), 50 ./ (g_learnt.eigvals(2):0.01:g_learnt.eigvals(end)) .^ 6, 'k',...
    'Linewidth', 2);
xlim([0 max(g_learnt.eigvals)]);
ylim(gpsd_limits);
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_learnt_knn.eigvals(2:end), diag(Gamma_learnt_knn(2:end, 2:end)), ...
    g_learnt_knn.eigvals(2):0.01:g_learnt_knn.eigvals(end), 1 ./ (g_learnt_knn.eigvals(2):0.01:g_learnt_knn.eigvals(end)) .^ 1, '--', ...
    'Linewidth', 2);
xlim([0 max(g_learnt_knn.eigvals)]);
ylim(gpsd_limits);
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_learnt_thr.eigvals(2:end), diag(Gamma_learnt_thr(2:end, 2:end)), ...
    g_learnt_thr.eigvals(2):0.01:g_learnt_thr.eigvals(end), 1 ./ (g_learnt_thr.eigvals(2):0.01:g_learnt_thr.eigvals(end)) .^ 1, '--', ...
    'Linewidth', 2);
xlim([0 max(g_learnt_thr.eigvals)]);
ylim(gpsd_limits);
title('Learnt THR', 'FontSize', 20);


%% Atoms responses

fh = grasp_open_figure_name('SGWT Atoms Responses');

ylimits = [0 1.5];

grasp_subaxis(fh, 2, 3, 1, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_plane.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_plane{m}(x), (1:size(C_sgwt_plane, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_plane.eigvals, zeros(numel(g_plane.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_plane.eigvals) max(g_plane.eigvals)]);
ylim(ylimits);
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 2, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_plane_knn.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_plane_knn{m}(x), (1:size(C_sgwt_plane_knn, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_plane_knn.eigvals, zeros(numel(g_plane_knn.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_plane_knn.eigvals) max(g_plane_knn.eigvals)]);
ylim(ylimits);
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 3, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_plane_thr.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_plane_thr{m}(x), (1:size(C_sgwt_plane_thr, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_plane_thr.eigvals, zeros(numel(g_plane_thr.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_plane_thr.eigvals) max(g_plane_thr.eigvals)]);
ylim(ylimits);
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 4, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_learnt.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_learnt{m}(x), (1:size(C_sgwt_learnt, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_learnt.eigvals, zeros(numel(g_learnt.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_learnt.eigvals) max(g_learnt.eigvals)]);
ylim(ylimits);
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 5, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_learnt_knn.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_learnt_knn{m}(x), (1:size(C_sgwt_learnt_knn, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_learnt_knn.eigvals, zeros(numel(g_learnt_knn.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_learnt_knn.eigvals) max(g_learnt_knn.eigvals)]);
ylim(ylimits);
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 6, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:(max(g_learnt_thr.eigvals));
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_learnt_thr{m}(x), (1:size(C_sgwt_learnt_thr, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_learnt_thr.eigvals, zeros(numel(g_learnt_thr.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_learnt_thr.eigvals) max(g_learnt_thr.eigvals)]);
ylim(ylimits);
title('Learnt THR', 'FontSize', 20);

%% Local PSD for the various graphs

lpsd_gauss = lpsd(Sigma_theory, G_sgwt_plane);
lpsd_gauss_knn = lpsd(Sigma_theory, G_sgwt_plane_knn);
lpsd_gauss_thr = lpsd(Sigma_theory, G_sgwt_plane_thr);

lpsd_learnt = lpsd(Sigma_theory, G_sgwt_learnt);
lpsd_learnt_knn = lpsd(Sigma_theory, G_sgwt_learnt_knn);
lpsd_learnt_thr = lpsd(Sigma_theory, G_sgwt_learnt_thr);

fprintf('Maximum Local PSD Power: %.2f dB\n',...
    10 * log(max([max(lpsd_gauss(:)) max(lpsd_gauss_knn(:)) max(lpsd_gauss_thr(:)) max(lpsd_learnt(:)) max(lpsd_learnt_knn(:)) max(lpsd_learnt_thr(:))])));

fh = grasp_open_figure_name('Theoretical local PSD');

caxis_lims = [-15 45];

SA_M = 0.0;
SA_P = 0.0;
SA_S = 0.0;
SA_PT = 0.04;
SA_MT = 0.05;
SA_SV = 0.01;

grasp_subaxis(fh, 2, 3, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_gauss));
caxis(caxis_lims);
colorbar;
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_gauss_knn));
caxis(caxis_lims);
colorbar;
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_gauss_thr));
caxis(caxis_lims);
colorbar;
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_learnt));
caxis(caxis_lims);
colorbar;
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_learnt_knn));
caxis(caxis_lims);
colorbar;
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_learnt_thr));
caxis(caxis_lims);
colorbar;
title('Learnt THR', 'FontSize', 20);

%% Mean Local PSD

grasp_open_figure_name('Theoretical Mean local PSD');
plot(0:30, 10 * log([...
        mean(lpsd_gauss, 2)...
        mean(lpsd_gauss_knn, 2)...
        mean(lpsd_gauss_thr, 2)...
        mean(lpsd_learnt, 2)...
        mean(lpsd_learnt_knn, 2)...
        mean(lpsd_learnt_thr, 2)]), '-*', 'LineWidth', 2)
xlabel('Scale');
ylabel('Power (dB)');
legend({'Gauss CMP', 'Gauss KNN', 'Gauss THR', 'Learnt CMP', 'Learnt KNN', 'Learnt THR'}, 'FontSize', 10)
