%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load Molène

g_mol = load('molene_data.mat');
g_mol = g_mol.molene_graphs{2};
g_mol.sigma = mean(g_mol.distances(g_mol.distances > 0)) / 3;
g_mol.A = grasp_adjacency_gaussian(g_mol, g_mol.sigma);

%% Some thresholding

g_mol_knn = g_mol;
g_mol_knn.KNN = 6;
g_mol_knn.A = grasp_adjacency_knn(g_mol, g_mol_knn.KNN) > 0;
g_mol_knn.A_layout = g_mol_knn.A;
g_mol_knn.sigma = mean(g_mol.distances(g_mol_knn.A > 0)) / 3;
g_mol_knn.A = g_mol_knn.A .* grasp_adjacency_gaussian(g_mol, g_mol_knn.sigma);
fprintf('Mean number of neighbors (KNN): %f\n', mean((g_mol_knn.A > 0) * ones(grasp_nb_nodes(g_mol), 1)));

g_mol_thr = g_mol;
% g_mol_thr.threshold = 0.4; % K=4
% g_mol_thr.threshold = 0.3; % K=5
g_mol_thr.threshold = 0.255; % K=6
% g_mol_thr.threshold = 0.078; % K=10
% g_mol_thr.threshold = 0.0415; % K=12
g_mol_thr.A = grasp_adjacency_thresh(g_mol, g_mol_thr.threshold) > 0;
g_mol_thr.A_layout = g_mol_thr.A;
g_mol_thr.sigma = mean(g_mol.distances(g_mol_thr.A > 0)) / 3;
g_mol_thr.A = g_mol_thr.A .* grasp_adjacency_gaussian(g_mol, g_mol_thr.sigma);
fprintf('Mean number of neighbors (THR): %f\n', mean((g_mol_thr.A > 0) * ones(grasp_nb_nodes(g_mol), 1)));

%% Local PSD from global correlation

% Unnormalized Atoms
% lpsd = @(R, G) reshape(diag(G * R * G'), size(G, 1) / size(G, 2), size(G, 2));

% Normalized Atoms
lpsd = @(R, G) reshape(diag(G * R * G') ./ diag(G * G'), size(G, 1) / size(G, 2), size(G, 2));

%% Harmonic Analysis

eig_matrix = 'std_lapl';

g_mol = grasp_eigendecomposition(g_mol, 'matrix', eig_matrix);
g_mol_knn = grasp_eigendecomposition(g_mol_knn, 'matrix', eig_matrix);
g_mol_thr = grasp_eigendecomposition(g_mol_thr, 'matrix', eig_matrix);

[G_sgwt_mol, C_sgwt_mol, C_sgwt_hat_mol, sgwt_resp_mol] = build_sgwt_atoms(g_mol, 15);
[G_sgwt_mol_knn, C_sgwt_mol_knn, C_sgwt_hat_mol_knn, sgwt_resp_mol_knn] = build_sgwt_atoms(g_mol_knn, size(C_sgwt_mol, 2) - 1, max(g_mol_knn.eigvals), 500);
[G_sgwt_mol_thr, C_sgwt_mol_thr, C_sgwt_hat_mol_thr, sgwt_resp_mol_thr] = build_sgwt_atoms(g_mol_thr, size(C_sgwt_mol, 2) - 1, max(g_mol_thr.eigvals), 500);

%% Data & Sample Correlation matrix

% Temperature increments
X = g_mol.data{1}(:, 2:end) - g_mol.data{1}(:, 1:(end - 1));
X_spatial_mean = mean(X, 1);

R = X * X' / size(X, 2);
Sigma = cov(X');
[C, C_pval] = corrcoef(X');

% RAW Temperatures
X_full = g_mol.data{1};
X_full_spa_mean = mean(X_full, 1);
X_full = X_full - repmat(X_full_spa_mean, [37, 1]);
X_full = X_full - repmat(mean(X_full, 2), [1 744]);
R_full = X_full * X_full' / size(X, 2);
Sigma_full = cov(X_full');
[C_full, C_full_pval] = corrcoef(X_full');

% To have the same analysis with the RAW data, uncomment the following
% block

% X = X_full;
% R = R_full;
% Sigma = Sigma_full;
% C = C_full;
% C_pval = C_full_pval;

grasp_open_figure_name('Spatial Mean');

SA_M = 0.05;
SA_P = 0.01;
SA_S = 0.1;
SA_MB = 0.05;

grasp_subaxis(gcf, 2, 2, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
plot((0:743) / 24, X_full_spa_mean);
xlabel('Time (days)');
xlim([0 32]);
title('Mean VS Time (RAW)', 'FontSize', 20);

grasp_subaxis(gcf, 2, 2, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
hist(X_full_spa_mean, 50);
xlabel(sprintf('Mean: %.2f / Std dev: %.2f', mean(X_full_spa_mean), std(X_full_spa_mean)));
title('Mean Distribution (RAW)', 'FontSize', 20);

grasp_subaxis(gcf, 2, 2, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
plot((0:742) / 24, X_spatial_mean);
xlabel('Time (days)');
xlim([0 32]);
title('Mean VS Time (INCR)', 'FontSize', 20);

grasp_subaxis(gcf, 2, 2, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
hist(X_spatial_mean, 50);
xlabel(sprintf('Mean: %f / Std dev: %f', mean(X_spatial_mean), std(X_spatial_mean)));
title('Mean Distribution (INCR)', 'FontSize', 20);

grasp_open_figure_name('Vertex Space Covariance');

SA_M = 0.05;
SA_P = 0.01;
SA_S = 0;
SA_MB = 0.1;

grasp_subaxis(gcf, 1, 2, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
imshow(Sigma_full);
caxis([0 max(abs(Sigma_full(:)))])
colorbar
xlabel(sprintf('\\mu_{C,offdiag} = %f\n', mean(abs(C_full(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Raw', 'FontSize', 20);

grasp_subaxis(gcf, 1, 2, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
imshow(Sigma);
caxis([0 max(abs(Sigma(:)))])
colorbar
xlabel(sprintf('\\mu_{C,offdiag} = %f\n', mean(abs(C(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Increments', 'FontSize', 20);

grasp_open_figure_name('Pearson Correlation VS Distance');

SA_M = 0.05;
SA_P = 0.01;
SA_S = 0.03;
SA_MB = 0.05;

grasp_subaxis(gcf, 1, 2, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
% Only the significant entries
semilogx(g_mol.distances(C_full_pval < 0.05), C_full(C_full_pval < 0.05), '.', 'MarkerSize', 20)
xlim([5 * 10^3 3 * 10^5])
ylim([0 1]);
title('Raw', 'FontSize', 20);

grasp_subaxis(gcf, 1, 2, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'MB', SA_MB);
% Only the significant entries
semilogx(g_mol.distances(C_pval < 0.05), C(C_pval < 0.05), '.', 'MarkerSize', 20)
xlim([5 * 10^3 3 * 10^5])
ylim([0 1]);
title('Increments', 'FontSize', 20);

%% Graph Learning

Lapl_learnt_data     = estimate_cgl(R, ones(grasp_nb_nodes(g_mol)) - eye(grasp_nb_nodes(g_mol)), 0, 1e-4, 1e-6, 100);
Lapl_learnt_data_knn = estimate_cgl(R, g_mol_knn.A > 0, 0, 1 * 1e-4, 1e-6, 100);
Lapl_learnt_data_thr = estimate_cgl(R, g_mol_thr.A > 0, 0, 1 * 1e-4, 1e-6, 100);

g_mol_learnt = g_mol;
g_mol_learnt.A = diag(diag(Lapl_learnt_data)) - Lapl_learnt_data;
g_mol_learnt = grasp_eigendecomposition(g_mol_learnt, 'matrix', eig_matrix);
[G_sgwt_mol_learnt, C_sgwt_mol_learnt, C_sgwt_hat_mol_learnt, sgwt_resp_mol_learnt] ...
    = build_sgwt_atoms(g_mol_learnt, size(C_sgwt_mol, 2) - 1, max(g_mol_learnt.eigvals), 20);

g_mol_learnt_knn = g_mol_knn;
g_mol_learnt_knn.A = diag(diag(Lapl_learnt_data_knn)) - Lapl_learnt_data_knn;
g_mol_learnt_knn = grasp_eigendecomposition(g_mol_learnt_knn, 'matrix', eig_matrix);
[G_sgwt_mol_learnt_knn, C_sgwt_mol_learnt_knn, C_sgwt_hat_mol_learnt_knn, sgwt_resp_mol_learnt_knn] ...
    = build_sgwt_atoms(g_mol_learnt_knn, size(C_sgwt_mol, 2) - 1, max(g_mol_learnt_knn.eigvals), 100);

g_mol_learnt_thr = g_mol_thr;
g_mol_learnt_thr.A = diag(diag(Lapl_learnt_data_thr)) - Lapl_learnt_data_thr;
g_mol_learnt_thr = grasp_eigendecomposition(g_mol_learnt_thr, 'matrix', eig_matrix);
[G_sgwt_mol_learnt_thr, C_sgwt_mol_learnt_thr, C_sgwt_hat_mol_learnt_thr, sgwt_resp_mol_learnt_thr] ...
    = build_sgwt_atoms(g_mol_learnt_thr, size(C_sgwt_mol, 2) - 1, max(g_mol_learnt_thr.eigvals), 100);

%% Plot the graph with the connectivity pattern

fh = grasp_open_figure_name('Molène Graph Weights');
SA_M = 0.01;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.03;
SA_MV = 0.03;
SA_SV = 0.01;

ah = grasp_subaxis(fh, 2, 3, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_mol;
ecs = [0 max(g_tmp.A(:))];
g_tmp.A_layout = g_tmp.A .* (g_tmp.A > 1 / 64);  % 64 is the number of colors of the colormaps in matlab
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Gauss CMP', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_mol_knn;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Gauss KNN', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_mol_thr;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Gauss THR', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_mol_learnt;
g_tmp.A_layout = g_tmp.A .* (g_tmp.A > 1 / 64);
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Learnt CMP', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_mol_learnt_knn;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Learnt KNN', 'FontSize', 20);

ah = grasp_subaxis(fh, 2, 3, 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV);
g_tmp = g_mol_learnt_thr;
g_tmp.A_layout = g_tmp.A;
grasp_show_graph(ah, g_tmp, 'edge_colormap', 'autumn', 'edge_thickness', 2, 'edge_color_scale', [0 1]);
title('Learnt THR', 'FontSize', 20);

%% Plot the eigenvalues

grasp_open_figure_name('Molène Eigenvalues'); clf;
plot(1:grasp_nb_nodes(g_mol), [g_mol.eigvals g_mol_knn.eigvals g_mol_thr.eigvals g_mol_learnt.eigvals g_mol_learnt_knn.eigvals g_mol_learnt_thr.eigvals], 'LineWidth', 2);
legend({'Gauss CMP', 'Gauss KNN', 'Gauss THR', 'Learnt CMP', 'Learnt KNN', 'Learnt THR'}, 'FontSize', 15)
ylim([0 g_mol_learnt.eigvals(end)]);
xlim([0 numel(g_mol_learnt.eigvals)]);
axes('Position', [.18 .5 .4 .4]);
plot(1:grasp_nb_nodes(g_mol), [g_mol.eigvals g_mol_knn.eigvals g_mol_thr.eigvals g_mol_learnt.eigvals g_mol_learnt_knn.eigvals g_mol_learnt_thr.eigvals], 'LineWidth', 1.5);
ylim([0 3]);
xlim([0 10]);

%% Some Fourier modes

sprintf('Minimum Fourier mode sample value: %f\n', ...
    [min(min(g_mol.Finv(:, 2:4)))...
    min(min(g_mol_knn.Finv(:, 2:4)))...
    min(min(g_mol_thr.Finv(:, 2:4)))...
    min(min(g_mol_learnt.Finv(:, 2:4)))...
    min(min(g_mol_learnt_knn.Finv(:, 2:4)))...
    min(min(g_mol_learnt_thr.Finv(:, 2:4)))]);

sprintf('Maximum Fourier mode sample value: %f\n', ...
    -[min(min(-g_mol.Finv(:, 2:4)))...
    min(min(-g_mol_knn.Finv(:, 2:4)))...
    min(min(-g_mol_thr.Finv(:, 2:4)))...
    min(min(-g_mol_learnt.Finv(:, 2:4)))...
    min(min(-g_mol_learnt_knn.Finv(:, 2:4)))...
    min(min(-g_mol_learnt_thr.Finv(:, 2:4)))]);

fh = grasp_open_figure_name('Mode #2,#3,#4 Comparison');
gft_freqs = [2 3 4];
% gft_freqs = [5 6 7];
% gft_freqs = [8 9 10];
% freqs = [198 199 200];
SA_M = 0.01;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.03;
SA_MV = 0.03;
SA_SV = 0.02;
SA_MB = 0.05;
val_scale_max = 0.7;
for cur_freq = 1:numel(gft_freqs)
    l = gft_freqs(cur_freq);
    
    ah = grasp_subaxis(fh, 3, 6, (cur_freq - 1) * 6 + 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV, 'MB', SA_MB);
    grasp_show_graph(ah, g_mol, 'show_edges', false, 'node_values', g_mol.Finv(:, l), 'value_scale', val_scale_max * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if cur_freq == 1
        title('Gauss CMP', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_mol.Finv(:, l)' * g_mol.L * g_mol.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 6, (cur_freq - 1) * 6 + 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV, 'MB', SA_MB);
    grasp_show_graph(ah, g_mol_knn, 'node_values', g_mol_knn.Finv(:, l), 'value_scale', val_scale_max * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if cur_freq == 1
        title('Gauss KNN', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_mol_knn.Finv(:, l)' * g_mol.L * g_mol_knn.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 6, (cur_freq - 1) * 6 + 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV, 'MB', SA_MB);
    grasp_show_graph(ah, g_mol_thr, 'node_values', g_mol_thr.Finv(:, l), 'value_scale', val_scale_max * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if cur_freq == 1
        title('Gauss THR', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_mol_thr.Finv(:, l)' * g_mol.L * g_mol_thr.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 6, (cur_freq - 1) * 6 + 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV, 'MB', SA_MB);
    grasp_show_graph(ah, g_mol_learnt, 'show_edges', false, 'node_values', g_mol_learnt.Finv(:, l), 'value_scale', val_scale_max * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if cur_freq == 1
        title('Learnt CMP', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_mol_learnt.Finv(:, l)' * g_mol.L * g_mol_learnt.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 6, (cur_freq - 1) * 6 + 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV, 'MB', SA_MB);
    grasp_show_graph(ah, g_mol_learnt_knn, 'node_values', g_mol_learnt_knn.Finv(:, l), 'value_scale', val_scale_max * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if cur_freq == 1
        title('Learnt KNN', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_mol_learnt_knn.Finv(:, l)' * g_mol.L * g_mol_learnt_knn.Finv(:, l)), 'FontSize', 15);
    
    ah = grasp_subaxis(fh, 3, 6, (cur_freq - 1) * 6 + 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MV', SA_MV, 'SV', SA_SV, 'MB', SA_MB);
    grasp_show_graph(ah, g_mol_learnt_thr, 'node_values', g_mol_learnt_thr.Finv(:, l), 'value_scale', val_scale_max * [-1 1]);
    grasp_set_blue_red_colormap(ah);
    if cur_freq == 1
        title('Learnt THR', 'FontSize', 20);
    end
    xlabel(sprintf('x^*Lx=%.3f', g_mol_learnt_thr.Finv(:, l)' * g_mol.L * g_mol_learnt_thr.Finv(:, l)), 'FontSize', 15);
end


%% Spectral Correlation & Correlation Coefficient Matrices for the various graphs

Gamma_gauss_mol = g_mol.F * R * g_mol.Finv;
gamma_gauss_mol = diag(Gamma_gauss_mol);
Gamma_gauss_mol_coeff = diag(1 ./ sqrt(gamma_gauss_mol)) * Gamma_gauss_mol * diag(1 ./ sqrt(gamma_gauss_mol));

Gamma_gauss_mol_knn = g_mol_knn.F * R * g_mol_knn.Finv;
gamma_gauss_mol_knn = diag(Gamma_gauss_mol_knn);
Gamma_gauss_mol_knn_coeff = diag(1 ./ sqrt(gamma_gauss_mol_knn)) * Gamma_gauss_mol_knn * diag(1 ./ sqrt(gamma_gauss_mol_knn));

Gamma_gauss_mol_thr = g_mol_thr.F * R * g_mol_thr.Finv;
gamma_gauss_mol_thr = diag(Gamma_gauss_mol_thr);
Gamma_gauss_mol_thr_coeff = diag(1 ./ sqrt(gamma_gauss_mol_thr)) * Gamma_gauss_mol_thr * diag(1 ./ sqrt(gamma_gauss_mol_thr));

Gamma_learnt_mol = g_mol_learnt.F * R * g_mol_learnt.Finv;
gamma_learnt_mol = diag(Gamma_learnt_mol);
Gamma_learnt_mol_coeff = diag(1 ./ sqrt(gamma_learnt_mol)) * Gamma_learnt_mol * diag(1 ./ sqrt(gamma_learnt_mol));

Gamma_learnt_mol_knn = g_mol_learnt_knn.F * R * g_mol_learnt_knn.Finv;
gamma_learnt_mol_knn = diag(Gamma_learnt_mol_knn);
Gamma_learnt_mol_knn_coeff = diag(1 ./ sqrt(gamma_learnt_mol_knn)) * Gamma_learnt_mol_knn * diag(1 ./ sqrt(gamma_learnt_mol_knn));

Gamma_learnt_mol_thr = g_mol_learnt_thr.F * R * g_mol_learnt_thr.Finv;
gamma_learnt_mol_thr = diag(Gamma_learnt_mol_thr);
Gamma_learnt_mol_thr_coeff = diag(1 ./ sqrt(gamma_learnt_mol_thr)) * Gamma_learnt_mol_thr * diag(1 ./ sqrt(gamma_learnt_mol_thr));


fh = grasp_open_figure_name('Molène Pearson Correlations');
max_corr = 100;
caxis_scale = [0 0.5];
SA_M = 0.01;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.02;
SA_PB = 0.1;

grasp_subaxis(gcf, 1, 7, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'PB', SA_PB);
imshow(C);
caxis(caxis_scale);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(C(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Increments', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'PB', SA_PB);
imshow(Gamma_gauss_mol_coeff);
caxis(caxis_scale);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_gauss_mol_coeff(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'PB', SA_PB);
imshow(Gamma_gauss_mol_knn_coeff);
caxis(caxis_scale);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_gauss_mol_knn_coeff(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'PB', SA_PB);
imshow(Gamma_gauss_mol_thr_coeff);
caxis(caxis_scale);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_gauss_mol_thr_coeff(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'PB', SA_PB);
imshow(Gamma_learnt_mol_coeff);
caxis(caxis_scale);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_learnt_mol_coeff(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'PB', SA_PB);
imshow(Gamma_learnt_mol_knn_coeff);
caxis(caxis_scale);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_learnt_mol_knn_coeff(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 7, 7, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'PB', SA_PB);
imshow(Gamma_learnt_mol_thr_coeff);
caxis(caxis_scale);
xlabel(sprintf('\\mu_{offdiag} = %f\n', mean(abs(Gamma_learnt_mol_thr_coeff(:))) - 1 / grasp_nb_nodes(g_mol)), 'FontSize', 15);
title('Learnt THR', 'FontSize', 20);

%% gPSD

%Note: the first graph frequency is removed otherwise log(0)=-infinity

fh = grasp_open_figure_name('Molène gPSD for various graphs');

gpsd_limits = [1.0 * 10^-1 1.2 * 10^0];
SA_M = 0.01;
SA_P = 0.01;
SA_S = 0;
SA_PT = 0.03;
SA_MT = 0.08;
SA_MB = 0.06;

grasp_subaxis(fh, 1, 6, 1, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_mol.eigvals(2:end), diag(Gamma_gauss_mol(2:end, 2:end)), '-x', ...
    g_mol.eigvals(2):0.01:g_mol.eigvals(end), .8 ./ (g_mol.eigvals(2):0.01:g_mol.eigvals(end)) .^ (0.6), '--',...
    g_mol.eigvals(2):0.01:g_mol.eigvals(end), 1 ./ (g_mol.eigvals(2):0.01:g_mol.eigvals(end)) .^ (1), '-.',...
    'Linewidth', 2);
xlim([0 max(g_mol.eigvals)]);
ylim(gpsd_limits);
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 2, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_mol_knn.eigvals(2:end), diag(Gamma_gauss_mol_knn(2:end, 2:end)), '-x', ...
    g_mol_knn.eigvals(2):0.01:g_mol_knn.eigvals(end), .2 ./ (g_mol_knn.eigvals(2):0.01:g_mol_knn.eigvals(end)) .^ (0.4), '--',...
    g_mol_knn.eigvals(2):0.01:g_mol_knn.eigvals(end), .1 ./ (g_mol_knn.eigvals(2):0.01:g_mol_knn.eigvals(end)) .^ (1), '-.',...
    'Linewidth', 2);
xlim([0 max(g_mol_knn.eigvals)]);
ylim(gpsd_limits);
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 3, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_mol_thr.eigvals(2:end), diag(Gamma_gauss_mol_thr(2:end, 2:end)), '-x', ...
    g_mol_thr.eigvals(2):0.01:g_mol_thr.eigvals(end), .2 ./ (g_mol_thr.eigvals(2):0.01:g_mol_thr.eigvals(end)) .^ (.3), '--', ...
    g_mol_thr.eigvals(2):0.01:g_mol_thr.eigvals(end), .1 ./ (g_mol_thr.eigvals(2):0.01:g_mol_thr.eigvals(end)) .^ (1), '-.',...
    'Linewidth', 2);
xlim([0 max(g_mol_thr.eigvals)]);
ylim(gpsd_limits);
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 4, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_mol_learnt.eigvals(2:end), diag(Gamma_learnt_mol(2:end, 2:end)), '-x', ...
    g_mol_learnt.eigvals(2):0.01:g_mol_learnt.eigvals(end), 1.5 ./ (g_mol_learnt.eigvals(2):0.01:g_mol_learnt.eigvals(end)) .^ 1, '--', ...
    'Linewidth', 2);
xlim([0 max(g_mol_learnt.eigvals)]);
ylim(gpsd_limits);
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 5, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_mol_learnt_knn.eigvals(2:end), diag(Gamma_learnt_mol_knn(2:end, 2:end)), '-x', ...
    g_mol_learnt_knn.eigvals(2):0.01:g_mol_learnt_knn.eigvals(end), 0.7 ./ (g_mol_learnt_knn.eigvals(2):0.01:g_mol_learnt_knn.eigvals(end)) .^ 0.6, '--', ...
    g_mol_learnt_knn.eigvals(2):0.01:g_mol_learnt_knn.eigvals(end), 1 ./ (g_mol_learnt_knn.eigvals(2):0.01:g_mol_learnt_knn.eigvals(end)) .^ (1), '-.',...
    'Linewidth', 2);
xlim([0 max(g_mol_learnt_knn.eigvals)]);
ylim(gpsd_limits);
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 1, 6, 6, 'M', SA_M, 'P', SA_P, 'S', SA_S, 'PT', SA_PT, 'MT', SA_MT, 'MB', SA_MB);
loglog(g_mol_learnt_thr.eigvals(2:end), diag(Gamma_learnt_mol_thr(2:end, 2:end)), '-x', ...
    g_mol_learnt_thr.eigvals(2):0.01:g_mol_learnt_thr.eigvals(end), 0.7 ./ (g_mol_learnt_thr.eigvals(2):0.01:g_mol_learnt_thr.eigvals(end)) .^ 0.6, '--', ...
    g_mol_learnt_thr.eigvals(2):0.01:g_mol_learnt_thr.eigvals(end), 1 ./ (g_mol_learnt_thr.eigvals(2):0.01:g_mol_learnt_thr.eigvals(end)) .^ 1, '-.', ...
    'Linewidth', 2);
xlim([0 max(g_mol_learnt_thr.eigvals)]);
ylim(gpsd_limits);
title('Learnt THR', 'FontSize', 20);


%% Atoms responses

fh = grasp_open_figure_name('Molène SGWT Atoms Responses');

ylimits = [0 1.5];

grasp_subaxis(fh, 2, 3, 1, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_mol.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_mol{m}(x), (1:size(C_sgwt_mol, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_mol.eigvals, zeros(numel(g_mol.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_mol.eigvals) max(g_mol.eigvals)]);
ylim(ylimits);
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 2, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_mol_knn.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_mol_knn{m}(x), (1:size(C_sgwt_mol_knn, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_mol_knn.eigvals, zeros(numel(g_mol_knn.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_mol_knn.eigvals) max(g_mol_knn.eigvals)]);
ylim(ylimits);
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 3, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_mol_thr.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_mol_thr{m}(x), (1:size(C_sgwt_mol_thr, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_mol_thr.eigvals, zeros(numel(g_mol_thr.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_mol_thr.eigvals) max(g_mol_thr.eigvals)]);
ylim(ylimits);
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 4, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_mol_learnt.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_mol_learnt{m}(x), (1:size(C_sgwt_mol_learnt, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_mol_learnt.eigvals, zeros(numel(g_mol_learnt.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_mol_learnt.eigvals) max(g_mol_learnt.eigvals)]);
ylim(ylimits);
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 5, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:max(g_mol_learnt_knn.eigvals);
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_mol_learnt_knn{m}(x), (1:size(C_sgwt_mol_learnt_knn, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_mol_learnt_knn.eigvals, zeros(numel(g_mol_learnt_knn.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_mol_learnt_knn.eigvals) max(g_mol_learnt_knn.eigvals)]);
ylim(ylimits);
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 6, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', 0.1, 'MB', 0.05, 'SV', 0.03);
x = 0:0.01:(max(g_mol_learnt_thr.eigvals));
plot(x, cell2mat(arrayfun(@(m) sgwt_resp_mol_learnt_thr{m}(x), (1:size(C_sgwt_mol_learnt_thr, 2))', 'UniformOutput', false)), 'Linewidth', 1.5);
hold on;
plot(g_mol_learnt_thr.eigvals, zeros(numel(g_mol_learnt_thr.eigvals), 1), 'kx', 'LineWidth', 3, 'MarkerSize',10);
hold off;
xlim([min(g_mol_learnt_thr.eigvals) max(g_mol_learnt_thr.eigvals)]);
ylim(ylimits);
title('Learnt THR', 'FontSize', 20);


%% Local PSD for the various graphs

lpsd_mol = lpsd(R, G_sgwt_mol);
lpsd_mol_knn = lpsd(R, G_sgwt_mol_knn);
lpsd_mol_thr = lpsd(R, G_sgwt_mol_thr);

lpsd_mol_learnt = lpsd(R, G_sgwt_mol_learnt);
lpsd_mol_learnt_knn = lpsd(R, G_sgwt_mol_learnt_knn);
lpsd_mol_learnt_thr = lpsd(R, G_sgwt_mol_learnt_thr);

maxi_psd = max([max(lpsd_mol(:)) max(lpsd_mol_knn(:)) max(lpsd_mol_thr(:)) max(lpsd_mol_learnt(:)) max(lpsd_mol_learnt_knn(:)) max(lpsd_mol_learnt_thr(:))]);
mini_psd = min([min(lpsd_mol(:)) min(lpsd_mol_knn(:)) min(lpsd_mol_thr(:)) min(lpsd_mol_learnt(:)) min(lpsd_mol_learnt_knn(:)) min(lpsd_mol_learnt_thr(:))]);
fprintf('Optimal Scale: [%f, %f]\n', 10 * log([mini_psd maxi_psd]));

fh = grasp_open_figure_name('Molène local PSD');

caxis_lims = [-21 21];

SA_PT = 0.05;
SA_MT = 0.02;
SA_SV = 0.02;

grasp_subaxis(fh, 2, 3, 1, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_mol));
caxis(caxis_lims);
colorbar;
title('Gauss CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 2, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_mol_knn));
caxis(caxis_lims);
colorbar;
title('Gauss KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 3, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_mol_thr));
caxis(caxis_lims);
colorbar;
title('Gauss THR', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 4, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_mol_learnt));
caxis(caxis_lims);
colorbar;
title('Learnt CMP', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 5, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_mol_learnt_knn));
caxis(caxis_lims);
colorbar;
title('Learnt KNN', 'FontSize', 20);

grasp_subaxis(fh, 2, 3, 6, 'M', 0.01, 'P', 0.01, 'S', 0, 'PT', SA_PT, 'MT', SA_MT, 'SV', SA_SV);
imshow(10 * log(lpsd_mol_learnt_thr));
caxis(caxis_lims);
colorbar;
title('Learnt THR', 'FontSize', 20);

%% Mean Local PSD

grasp_open_figure_name('Molène Mean local PSD');
plot(0:(size(C_sgwt_mol, 2) - 1), 10 * log([...
        mean(lpsd_mol, 2)...
        mean(lpsd_mol_knn, 2)...
        mean(lpsd_mol_thr, 2)...
        mean(lpsd_mol_learnt, 2)...
        mean(lpsd_mol_learnt_knn, 2)...
        mean(lpsd_mol_learnt_thr, 2)]), '-*', 'LineWidth', 2)
ylim([-16 21]);
xlabel('Scale');
ylabel('Power (dB)');
legend({'Gauss CMP', 'Gauss KNN', 'Gauss THR', 'Learnt CMP', 'Learnt KNN', 'Learnt THR'}, 'FontSize', 10)


